"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
Object.defineProperty(exports, "__esModule", { value: true });
var common_tags_1 = require("common-tags");
// having weak reference to styles prevents garbage collection
// and "losing" styles when the next test starts
var stylesCache = new Map();
var copyStyles = function (component) {
    // need to find same component when component is recompiled
    // by the JSX preprocessor. Thus have to use something else,
    // like component name
    // const hash = component.type.name
    var hash = component;
    var styles = document.querySelectorAll('head style');
    if (styles.length) {
        console.log('injected %d styles', styles.length);
        stylesCache.set(hash, styles);
    }
    else {
        console.log('No styles injected for this component, checking cache');
        if (stylesCache.has(hash)) {
            styles = stylesCache.get(hash);
        }
        else {
            styles = null;
        }
    }
    if (!styles) {
        return;
    }
    var parentDocument = window.parent.document;
    var projectName = Cypress.config('projectName');
    var appIframeId = "Your App: '" + projectName + "'";
    var appIframe = parentDocument.getElementById(appIframeId);
    var head = appIframe.contentDocument.querySelector('head');
    styles.forEach(function (style) {
        head.appendChild(style);
    });
};
function setXMLHttpRequest(w) {
    // by grabbing the XMLHttpRequest from app's iframe
    // and putting it here - in the test iframe
    // we suddenly get spying and stubbing 😁
    window.XMLHttpRequest = w.XMLHttpRequest;
    return w;
}
function setAlert(w) {
    window.alert = w.alert;
    return w;
}
exports.mount = function (state, actions, view) {
    if (!actions) {
        // we always want to have an actions object so we
        // can attach _getState utility function
        actions = {};
    }
    // TODO stop hard coding Hyperapp version, grab from the "node_modules"
    var html = common_tags_1.stripIndent(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n    <head>\n      <meta charset=\"UTF-8\">\n    </head>\n    <body>\n      <div id=\"app\"></div>\n      <script src=\"https://unpkg.com/hyperapp@1.2.0/dist/hyperapp.js\"></script>\n    </body>\n  "], ["\n    <head>\n      <meta charset=\"UTF-8\">\n    </head>\n    <body>\n      <div id=\"app\"></div>\n      <script src=\"https://unpkg.com/hyperapp@1.2.0/dist/hyperapp.js\"></script>\n    </body>\n  "])));
    var document = cy.state('document');
    document.write(html);
    document.close();
    // add a utility action to get the current state
    if (!actions._getState) {
        var _getState = function () { return function (state) { return state; }; };
        actions = Object.assign({}, actions, { _getState: _getState });
    }
    // force waiting for window.hyperapp to exist before anything else happens
    cy
        .window({ log: false })
        .its('hyperapp')
        .should('be.an', 'object');
    // now can attach our handlers
    cy
        .window({ log: false })
        .then(setXMLHttpRequest)
        .then(setAlert)
        .its('hyperapp.app')
        .then(function (app) {
        var el = document.getElementById('app');
        var main = app(state, actions, view, el);
        // wrap every hyper action with `cy.then` to
        // make sure it goes through the Cypress command queue
        // allows things like the example below to just work
        //   Cypress.main.setText('foo')
        //   cy.contains('foo')
        //   Cypress.main.setText('bar')
        //   cy.contains('bar')
        Cypress.main = {};
        Object.keys(main).forEach(function (name) {
            var action = main[name];
            Cypress.main[name] = function queueAction() {
                // should we log arguments?
                cy.log("action: " + name);
                return cy.then(function () { return action.apply(null, arguments); });
            };
        });
        return Cypress.main;
    });
    cy.get('#app', { log: false }).should('be.visible');
    // picostyle dom is more complex
    // picostyle inserts a style into the document's head
    // but then it actually inserts new CSS rules when the view instantiates
    // so we need to _observe_ test document style and on added
    // CSS rules copy them
    copyStyles(view);
};
var templateObject_1;
